# 
# Define the class used to hold references to document, items within
# documents, and standard sites.
#

require "yaml"

class Reference
  
  ###
  ### Constructors and Destructors
  ###
  
  def initialize
    load_list
  end
  
  ###
  ### Accessors
  ###
  
  def acronym_list
    @acronym_list
  end
  
  ###
  ### Update methods. Used once we have all the data, but before
  ### we do any generating, to make sure everything is up-to-date
  ###
  
  def update(syntax_tree)
    # Update the document list
    
    # Walk the forest, looking for targets at the paragraph
    # level
    syntax_tree.block_forest.each{|tree|
      #walker.walk_tree(tree)
      #puts tree.content.target
      
      tree.each{|node|
        # Look for interesting nodes
        case node.content.type
          when :ac
            # Can we find this in the list of acronyms?
            unless @acronym_list.include?(node.content.content) then
              acronym = Hash.new
              acronym['text'] = node.content.content
              
              @acronym_list[node.content.content] = acronym
            end
        end
      }
    }
    
    # Save the updates
    save_list

  end
  
  ###
  ### Load and Store Helpers. Used to persit the reference list
  ### between compiles
  ###
  
  # Save 
  def save_list

    # Save the document list
    
    ##
    ## Save the list of acronyms
    ##
    
    # Does the file path exist
    unless Dir.exists?(File.dirname(@acronym_filename)) then
      FileUtils.mkdir_p(File.dirname(@acronym_filename))
    end
    
    File.open(@acronym_filename, 'w') do |out|
      YAML.dump(@acronym_list, out)
    end
    
  end
  
  # Load
  def load_list
    
    # Save the physical location of the reference directory
    @root = Dir.pwd + "/ref"
    
    # Load the list of acronyms
    @acronym_filename = @root + "/acronyms.yaml"
    if File.exists?(@acronym_filename) then
      @acronym_list = YAML.load_file(@acronym_filename)
    else
      @acronym_list = Hash.new
    end
        
  end
    
end

# Master instantiation
$reference = Reference.new